package models

import (
	"eta/eta_index_lib/utils"
	"fmt"
	"github.com/beego/beego/v2/client/orm"
	"strconv"
	"strings"
	"time"
)

// BaseFromSci99Index 代表卓创资讯-原始指标表的结构
type BaseFromSci99Index struct {
	BaseFromSciIndexId int       `orm:"column(base_from_sci_index_id);pk"` // 主键，自动递增
	IndexCode          string    // 指标编码
	IndexName          string    // 指标名称
	ClassifyId         int       // 分类Id
	Unit               string    // 单位
	Frequency          string    // 频度
	Describe           string    // 指标描述
	CreateTime         time.Time // 创建时间
	ModifyTime         time.Time // 修改时间
	StartDate          time.Time `gorm:"column:start_date"`
	EndDate            time.Time `gorm:"column:end_date"`
	LatestValue        float64   `gorm:"column:latest_value"` // 数据最新值
}

// BaseFromSci99Data 代表卓创资讯-原始指标数据表的结构
type BaseFromSci99DataItem struct {
	BaseFromSciDataId  int       `orm:"column(base_from_sci_data_id);pk"` // 主键，自动递增
	BaseFromSciIndexId int       // 指标id
	IndexCode          string    // 指标编码
	DataTime           string    // 数据日期
	Value              string    // 数据值
	CreateTime         time.Time // 创建时间
	ModifyTime         time.Time // 修改时间
}

// BaseFromSci99Data 代表卓创资讯-原始指标数据表的结构
type BaseFromSci99Data struct {
	BaseFromSciDataId  int       `orm:"column(base_from_sci_data_id);pk"` // 主键，自动递增
	BaseFromSciIndexId int       // 指标id
	IndexCode          string    // 指标编码
	DataTime           string    // 数据日期
	Value              float64   // 数据值
	CreateTime         time.Time // 创建时间
	ModifyTime         time.Time // 修改时间
}

// BaseFromSci99Classify 代表卓创资讯-原始指标分类表的结构
type BaseFromSci99Classify struct {
	BaseFromSciClassifyId int       `orm:"column(base_from_sci_classify_id);pk"` // 主键，自动递增
	ClassifyName          string    // 分类名称
	Sort                  int       // 排序
	CreateTime            time.Time // 创建时间
	ModifyTime            time.Time // 修改时间
}

func GetBaseFromSci99DataDataByCondition(condition string, pars []interface{}) (item []*BaseFromIcpiData, err error) {
	sql := ` SELECT * FROM base_from_sci99_data WHERE 1=1 `
	o := orm.NewOrm()
	if condition != "" {
		sql += condition
	}
	sql += ` ORDER BY data_time DESC `
	_, err = o.Raw(sql, pars).QueryRows(&item)
	return
}

// 添加数据
func AddBaseFromSci99Index(item *BaseFromSci99Index) (lastId int64, err error) {
	o := orm.NewOrm()
	lastId, err = o.Insert(item)
	return
}

func AddBaseFromSci99Classify(item *BaseFromSci99Classify) (lastId int64, err error) {
	o := orm.NewOrm()
	lastId, err = o.Insert(item)
	return
}

func AddBaseFromSci99DataMulti(item []*BaseFromSci99Data) (err error) {
	o := orm.NewOrm()
	_, err = o.InsertMulti(1000, item)
	return
}

func GetBaseFromSci99Index() (list []*BaseFromSci99Index, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM base_from_sci99_index `
	_, err = o.Raw(sql).QueryRows(&list)
	return
}

func GetBaseFromSci99DataByIndexCode(indexCode string) (items []*BaseFromSci99DataItem, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM base_from_sci99_data WHERE index_code=? `
	_, err = o.Raw(sql, indexCode).QueryRows(&items)
	return
}

// 新增卓创资讯指标数据
func AddEdbDataFromSci99(edbCode string) (err error) {
	o := orm.NewOrm()
	dataAll, err := GetBaseFromSci99DataByIndexCode(edbCode)
	if err != nil && err.Error() != utils.ErrNoRow() {
		return
	}

	var isAdd bool
	addSql := ` INSERT INTO edb_data_sci99(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	existMap := make(map[string]string)

	for _, sv := range dataAll {
		eDate := sv.DataTime
		dataTime, err := time.ParseInLocation(utils.FormatDate, eDate, time.Local)
		if err != nil {
			fmt.Println("time.Parse Err:" + eDate)
			return err
		}
		timestamp := dataTime.UnixNano() / 1e6
		timeStr := fmt.Sprintf("%d", timestamp)
		if _, ok := existMap[eDate]; !ok {
			addSql += GetAddSql("0", edbCode, eDate, timeStr, sv.Value)
			isAdd = true
		}
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		utils.FileLog.Info("addSql:" + addSql)
		_, err = o.Raw(addSql).Exec()
		if err != nil {
			return err
		}
	}
	return
}

// 刷新卓创资讯指标数据
func RefreshEdbDataFromSci99(edbInfoId int, edbCode, startDate string) (err error) {
	source := utils.DATA_SOURCE_SCI99
	subSource := utils.DATA_SUB_SOURCE_EDB

	o := orm.NewOrm()
	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}

	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}

	dataList, err := GetBaseFromSci99DataDataByCondition(condition, pars)
	if err != nil {
		return
	}

	// 真实数据的最大日期  ， 插入规则配置的日期
	var realDataMaxDate, edbDataInsertConfigDate time.Time
	var edbDataInsertConfig *EdbDataInsertConfig
	var isFindConfigDateRealData bool //是否找到配置日期的实际数据的值
	{
		edbDataInsertConfig, err = GetEdbDataInsertConfigByEdbId(edbInfoId)
		if err != nil && err.Error() != utils.ErrNoRow() {
			return
		}
		if edbDataInsertConfig != nil {
			edbDataInsertConfigDate = edbDataInsertConfig.Date
		}
	}

	//获取指标所有数据
	var existCondition string
	var existPars []interface{}

	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)
	if startDate != "" {
		existCondition += " AND data_time>=? "
		existPars = append(existPars, startDate)
	}

	existList, err := GetEdbDataByCondition(source, subSource, existCondition, existPars)
	if err != nil {
		return err
	}
	existMap := make(map[string]*EdbInfoSearchData)
	for _, v := range existList {
		existMap[v.DataTime] = v
	}

	addSql := ` INSERT INTO edb_data_sci99(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	var isAdd bool
	for _, v := range dataList {
		item := v
		itemValue := v.Value
		eDate := item.DataTime
		dataTime, err := time.ParseInLocation(utils.FormatDate, eDate, time.Local)
		if err != nil {
			return err
		}
		if _, ok := existMap[v.DataTime]; !ok {
			sValue := itemValue
			if sValue != "" {
				timestamp := dataTime.UnixNano() / 1e6
				timeStr := fmt.Sprintf("%d", timestamp)
				saveValue := sValue

				if findItem, ok := existMap[eDate]; !ok {
					addSql += GetAddSql(edbInfoIdStr, edbCode, eDate, timeStr, saveValue)
					isAdd = true
				} else {
					if findItem != nil && utils.SubFloatToString(findItem.Value, 30) != sValue {
						err = ModifyEdbDataById(source, subSource, findItem.EdbDataId, sValue)
						if err != nil {
							return err
						}
					}
				}
			}
		}

		// 下面代码主要目的是处理掉手动插入的数据判断
		{
			if realDataMaxDate.IsZero() || dataTime.After(realDataMaxDate) {
				realDataMaxDate = dataTime
			}
			if edbDataInsertConfigDate.IsZero() || dataTime.Equal(edbDataInsertConfigDate) {
				isFindConfigDateRealData = true
			}
		}
	}

	// 处理手工数据补充的配置
	HandleConfigInsertEdbData(realDataMaxDate, edbDataInsertConfig, edbInfoId, source, subSource, existMap, isFindConfigDateRealData)

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = o.Raw(addSql).Exec()
		if err != nil {
			return err
		}
	}
	return
}

func GetBaseFromSci99Classify() (list []*BaseFromSci99Classify, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM base_from_sci99_classify `
	_, err = o.Raw(sql).QueryRows(&list)
	return
}

func GetBaseFromTradeSci99DataAll(indexCode string) (list []*BaseFromSci99Data, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM base_from_sci99_data where index_code=?`
	_, err = o.Raw(sql, indexCode).QueryRows(&list)
	return
}
func GetBaseFromTradeSci99LatestData(indexCode string) (date time.Time, err error) {
	o := orm.NewOrm()
	sql := `SELECT data_time FROM base_from_sci99_data where index_code=? ORDER BY data_time DESC limit 1 `
	err = o.Raw(sql, indexCode).QueryRow(&date)
	return
}

// UpdateBaseFromSci99Data
func UpdateBaseFromSci99Data(value float64, indexCode, dataTime string) (err error) {
	o := orm.NewOrm()
	sql := `UPDATE base_from_sci99_data SET value=?,modify_time=NOW() WHERE index_code = ? AND data_time = ? `
	_, err = o.Raw(sql, value, indexCode, dataTime).Exec()
	return
}

func (m *BaseFromSci99Index) GetMaxAndMinDateByIndexCode(indexCode string) (item *EdbInfoMaxAndMinInfo, err error) {
	o := orm.NewOrm()
	sql := ` SELECT MIN(data_time) AS min_date,MAX(data_time) AS max_date,MIN(value) AS min_value,MAX(value) AS max_value FROM base_from_sci99_data WHERE index_code=? `
	err = o.Raw(sql, indexCode).QueryRow(&item)
	if err != nil {
		return
	}

	// 获取最新值
	var latest_value float64
	sql = ` SELECT value AS latest_value FROM base_from_sci99_data WHERE index_code=? ORDER BY data_time DESC LIMIT 1 `
	err = o.Raw(sql, indexCode).QueryRow(&latest_value)
	if err != nil {
		return
	}
	item.LatestValue = latest_value

	return
}

func (m *BaseFromSci99Index) ModifyIndexMaxAndMinDate(indexCode string, item *EdbInfoMaxAndMinInfo) (err error) {
	o := orm.NewOrm()
	sql := ` UPDATE base_from_sci99_index SET start_date=?,end_date=?,latest_value=?,modify_time=NOW() WHERE index_code=? `
	_, err = o.Raw(sql, item.MinDate, item.MaxDate, item.LatestValue, indexCode).Exec()
	return
}
